////////////////////////////////////////
// Read reconstruction ROOT output file
// Plot variables
////////////////////////////////////////

#include "ROOT/RDataFrame.hxx"
#include <iostream>

#include "dd4pod/Geant4ParticleCollection.h"
#include "eicd/ClusterCollection.h"
#include "eicd/ClusterData.h"

#include "TCanvas.h"
#include "TStyle.h"
#include "TMath.h"
#include "TH1.h"
#include "TH1D.h"

// If root cannot find a dd4pod header make sure to add the include dir to ROOT_INCLUDE_PATH:
//  export ROOT_INCLUDE_PATH=$HOME/include:$ROOT_INCLUDE_PATH
//  export ROOT_INCLUDE_PATH=/home/jihee/stow/development/include:$ROOT_INCLUDE_PATH

using ROOT::RDataFrame;
using namespace ROOT::VecOps;

void full_emcal_electrons_analysis(const char* input_fname = "rec_emcal_uniform_electrons.root")
{
  // Setting for graphs
  gROOT->SetStyle("Plain");
  gStyle->SetOptFit(1);
  gStyle->SetLineWidth(2);
  gStyle->SetPadTickX(1);
  gStyle->SetPadTickY(1);
  gStyle->SetPadGridX(1);
  gStyle->SetPadGridY(1);
  gStyle->SetPadLeftMargin(0.14);
  gStyle->SetPadRightMargin(0.14);

  ROOT::EnableImplicitMT();
  ROOT::RDataFrame d0("events", input_fname);

  // Thrown Energy [GeV]
  auto E_thr = [](std::vector<dd4pod::Geant4ParticleData> const& input) {
  	std::vector<float> result;
    	result.push_back(TMath::Sqrt(input[2].psx*input[2].psx + input[2].psy*input[2].psy + input[2].psz*input[2].psz + input[2].mass*input[2].mass));
  return result;
  };

  // Define variables
  auto d1 = d0.Define("E_thr",    E_thr,    {"mcparticles2"})
	      ;

  // Define Histograms
  auto hEthr     = d1.Histo1D({"hEthr",     "Thrown Energy; E_{thr}[GeV]; Events",          100, -0.5, 30.5},   "E_thr");

  // Event Counts
  auto nevents_thrown    = d1.Count();

  // Print out number of events
  std::cout << "Number of Events: " << (*nevents_thrown) << " = all thrown events \n";

  // Draw Histogram
  TCanvas *c1 = new TCanvas("c1", "c1", 500, 500);
  hEthr->GetYaxis()->SetTitleOffset(1.4);
  hEthr->SetLineWidth(2);
  hEthr->SetLineColor(kBlue);
  hEthr->DrawClone();
  c1->SaveAs("results/emcal_electrons_Ethr.png");
  c1->SaveAs("results/emcal_electrons_Ethr.pdf");
}
