int read_eng(const char* output_fname = "sim_output/read_eng_output.root", const char *fname = "results/eng.txt")
{
  // Input ASCII File
  FILE *fin = fopen(fname,"r");

  if(!fin)
	fprintf(stderr,"error: can't open %s\n", fname);

  // Output ROOT File
  TFile *rootFile = new TFile(output_fname,"recreate");
  // ROOT tree that goes into this ROOT file
  TTree *tEngRes = new TTree("tEngRes","Energy Resolution Information");
  // Variables that go into ROOT file
  Int_t    ievent;
  Double_t tot_clust_eng; // [GeV]
  Int_t    ncluster;
  Double_t mc_eng;        // [GeV]
  Double_t sim_tru_eng;   // [GeV]
  Double_t sim_eng;       // [GeV]
  // Declare ROOT tree branches
  tEngRes->Branch("ievent",        &ievent,        "ievent/I");
  tEngRes->Branch("tot_clust_eng", &tot_clust_eng, "tot_clust_eng/D");
  tEngRes->Branch("ncluster",      &ncluster,      "ncluster/I");
  tEngRes->Branch("mc_eng",        &mc_eng,        "mc_eng/D");
  tEngRes->Branch("sim_tru_eng",   &sim_tru_eng,   "sim_tru_eng/D");
  tEngRes->Branch("sim_eng",       &sim_eng,       "sim_eng/D");

  char inbuf[1024];
  int  nevents_readout = 0;
  int  linenum = 0;

  TString s;
  TObjArray *o = 0;

  // Read each line
  while(fgets(inbuf,1024,fin))
  {
	nevents_readout++;
	s = inbuf;
	o = s.Tokenize(" \t\n");
	for(int icol = 0; icol < o->GetEntries(); icol++)
	{
		s = ((TObjString *)(*o)[icol])->GetString();
		if(icol==0)
			sscanf(s.Data(),"%d", &ievent);
		if(icol==1)
			sscanf(s.Data(),"%lf", &mc_eng);
		if(icol==2)
			sscanf(s.Data(),"%lf", &sim_tru_eng);
		if(icol==3)
			sscanf(s.Data(),"%lf", &sim_eng);
		if(icol==5)
			sscanf(s.Data(),"%d", &ncluster);
		if(icol==6)
			sscanf(s.Data(),"%lf", &tot_clust_eng);
	}
	tEngRes->Fill();
	// Clean up
	delete o;
  } 
  // Done reading ASCII file. So finialize the ROOT tree and close the ROOT file
  fprintf(stdout,"%d events read in!\n",nevents_readout);
  // Finalize ROOT file
  tEngRes->Write();
  // Close ROOT file
  rootFile->Close();
  fclose(fin);

  return 0;
}
