#!/bin/bash

./util/print_env.sh

## To run the reconstruction, we need the following global variables:
## - JUGGLER_INSTALL_PREFIX:   Install prefix for Juggler (simu/recon)
## - JUGGLER_DETECTOR:         the detector package we want to use for this benchmark
## - JUGGLER_DETECTOR_VERSION: the detector package we want to use for this benchmark
## - DETECTOR_PATH:            full path to the detector definitions
##
## You can ready options/env.sh for more in-depth explanations of the variables
## and how they can be controlled.
source options/env.sh
export DETECTOR_PATH=${DETECTOR_PATH}

if [[ ! -n  "${JUGGLER_N_EVENTS}" ]] ; then 
  export JUGGLER_N_EVENTS=100
fi

export JUGGLER_FILE_NAME_TAG="rich_forward_hadrons"
export JUGGLER_GEN_FILE="${JUGGLER_FILE_NAME_TAG}.hepmc"

export JUGGLER_SIM_FILE="sim_${JUGGLER_FILE_NAME_TAG}.root"
export JUGGLER_REC_FILE="rec_${JUGGLER_FILE_NAME_TAG}.root"

echo "JUGGLER_N_EVENTS = ${JUGGLER_N_EVENTS}"
echo "JUGGLER_DETECTOR = ${JUGGLER_DETECTOR}"


# generate the input events
# note datasets is now only used to develop datasets.
#git clone https://eicweb.phy.anl.gov/EIC/datasets.git datasets
python benchmarks/rich/scripts/rich_data_gen.py \
       ${JUGGLER_FILE_NAME_TAG}.hepmc \
       -n ${JUGGLER_N_EVENTS} \
       --pmin 5.0 \
       --pmax 100.0 \
       --angmin 3.0 \
       --angmax 8.0
if [[ "$?" -ne "0" ]] ; then
  echo "ERROR running script"
  exit 1
fi

# This script appears to be missing
## run geant4 simulations
#python benchmakrs/options/ForwardRICH/simu.py \
#       --compact=${DETECTOR_PATH}/${JUGGLER_DETECTOR}.xml \
#       -i ${JUGGLER_FILE_NAME_TAG}.hepmc \
#       -o ${JUGGLER_SIM_FILE}
#       -n ${JUGGLER_N_EVENTS}
#if [[ "$?" -ne "0" ]] ; then
#  echo "ERROR running script"
#  exit 1
#fi

#### skipping 
exit 0 


npsim --runType batch \
      --part.minimalKineticEnergy 1000*GeV  \
      -v WARNING \
      --numberOfEvents ${JUGGLER_N_EVENTS} \
      --compactFile ${DETECTOR_PATH}/${JUGGLER_DETECTOR}.xml \
      --inputFiles  ${JUGGLER_FILE_NAME_TAG}.hepmc \
      --outputFile  ${JUGGLER_SIM_FILE}
if [[ "$?" -ne "0" ]] ; then
  echo "ERROR running npdet"
  exit 1
fi

# @TODO changeable simulation file name and detector xml file name
xenv -x ${JUGGLER_INSTALL_PREFIX}/Juggler.xenv \
  gaudirun.py benchmarks/rich/options/rich_reco.py
if [[ "$?" -ne "0" ]] ; then
  echo "ERROR running juggler"
  exit 1
fi

# @TODO add analysis scripts
#root_filesize=$(stat --format=%s "${JUGGLER_DETECTOR}/${JUGGLER_REC_FILE}")
#if [[ "${JUGGLER_N_EVENTS}" -lt "500" ]] ; then 
#  # file must be less than 10 MB to upload
#  if [[ "${root_filesize}" -lt "10000000" ]] ; then 
#    cp ${JUGGLER_DETECTOR}/${JUGGLER_REC_FILE} results/.
#  fi
#fi

